package ai.nextbillion.octant.engine;

import android.app.PendingIntent;
import android.content.Context;
import android.location.Location;
import android.os.Looper;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.amap.api.location.AMapLocation;
import com.amap.api.location.AMapLocationClient;
import com.amap.api.location.AMapLocationClientOption;
import com.amap.api.location.AMapLocationListener;

public class AmapLocationEngineImpl implements LocationEngineImpl<AMapLocationListener> {
    AMapLocationClient aMapLocationClient;

    public AmapLocationEngineImpl(Context context) {
        AMapLocationClient.updatePrivacyShow(context, true, true);
        AMapLocationClient.updatePrivacyAgree(context, true);
        try {
            aMapLocationClient = new AMapLocationClient(context.getApplicationContext());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @NonNull
    @Override
    public AMapLocationListener createListener(LocationEngineCallback<LocationEngineResult> callback) {
        return new AmapLocationEngineCallbackTransport(callback);
    }

    @Override
    public void getLastLocation(@NonNull LocationEngineCallback<LocationEngineResult> callback) throws SecurityException {
        if (aMapLocationClient == null) {
            callback.onFailure(new Exception("aMapLocationClient is null"));
            return;
        }
        AMapLocation aMapLocation = aMapLocationClient.getLastKnownLocation();
        onAMapLocation(aMapLocation, callback);
    }

    @Override
    public void requestLocationUpdates(@NonNull LocationEngineRequest request, @NonNull AMapLocationListener listener, @Nullable Looper looper) throws SecurityException {
        aMapLocationClient.setLocationOption(config(request));
        aMapLocationClient.setLocationListener(listener);
        if (aMapLocationClient.isStarted()) {
            aMapLocationClient.stopLocation();
            aMapLocationClient.setLocationOption(config(request));
            aMapLocationClient.setLocationListener(listener);
        }
        aMapLocationClient.startLocation();
    }

    @Override
    public void requestLocationUpdates(@NonNull LocationEngineRequest request, @NonNull PendingIntent pendingIntent) throws SecurityException {

    }

    @Override
    public void removeLocationUpdates(AMapLocationListener listener) {
        aMapLocationClient.unRegisterLocationListener(listener);
        if (aMapLocationClient.isStarted()) {
            aMapLocationClient.stopLocation();
        }
    }

    @Override
    public void removeLocationUpdates(PendingIntent pendingIntent) {

    }

    static final class AmapLocationEngineCallbackTransport implements AMapLocationListener {

        private final LocationEngineCallback<LocationEngineResult> callback;

        public AmapLocationEngineCallbackTransport(LocationEngineCallback<LocationEngineResult> callback) {
            this.callback = callback;
        }


        @Override
        public void onLocationChanged(AMapLocation aMapLocation) {
            onAMapLocation(aMapLocation, callback);
        }
    }

    static void onAMapLocation(AMapLocation amapLocation, LocationEngineCallback<LocationEngineResult> callback) {
        if (amapLocation != null) {
            String errorInfo = amapLocation.getErrorInfo();
            int errorCode = amapLocation.getErrorCode();
            if (errorCode != 0) {
                callback.onFailure(new Exception("Amap locating errorCode:" + errorCode + " errorInfo:" + errorInfo));
            } else {
                callback.onSuccess(LocationEngineResult.create(convert(amapLocation)));
            }
        } else {
            callback.onFailure(new Exception("Amap returns null/empty location"));
        }
    }

    ///////////////////////////////////////////////////////////////////////////
    //
    ///////////////////////////////////////////////////////////////////////////

    static Location convert(AMapLocation aMapLocation) {
        Location location = new Location("Amap");
        location.setLatitude(aMapLocation.getLatitude());
        location.setLongitude(aMapLocation.getLongitude());
        location.setTime(aMapLocation.getTime());
        return location;
    }

    AMapLocationClientOption config(LocationEngineRequest request) {
        AMapLocationClientOption option = new AMapLocationClientOption();

        option.setLocationMode(AMapLocationClientOption.AMapLocationMode.Hight_Accuracy);
        option.setInterval(request.getInterval());
        option.setMockEnable(false);
        option.setNeedAddress(false);
        return option;
    }

    @Override
    public String toString() {
        return "AmapLocationEngineImpl{" +
                "aMapLocationClient=" + aMapLocationClient +
                '}';
    }
}
